﻿using System;
using System.Globalization;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Routing;

namespace ThinkGeo.MapSuite.RoutingExamples
{
    public class RouteDirectionModel
    {
        private static string directionImageUriTemplate = "../../Image/Directions/{0}";

        private string roadNameColumnName;
        private string directionImageUriString;
        private Feature roadFeature;
        private DistanceUnit lengthUnit;
        private RouteSegment roadSegment;
        private Feature roadSegmentFeature;
        private GeographyUnit geographyUnit;

        public RouteDirectionModel()
            : this(null, null, null)
        { }

        public RouteDirectionModel(RouteSegment roadSegment, Feature roadFeature, string roadNameColumnName)
            : this(roadSegment, roadFeature, roadNameColumnName, GeographyUnit.DecimalDegree, DistanceUnit.Meter)
        { }

        public RouteDirectionModel(RouteSegment roadSegment, Feature roadFeature, string roadNameColumnName, GeographyUnit geographyUnit, DistanceUnit lengthUnit)
        {
            this.roadSegment = roadSegment;
            this.roadFeature = roadFeature;
            this.roadNameColumnName = roadNameColumnName;
            this.geographyUnit = geographyUnit;
            this.lengthUnit = lengthUnit;
        }

        public DrivingDirection Direction
        {
            get { return roadSegment != null ? roadSegment.DrivingDirection : DrivingDirection.Back; }
        }

        public GeographyUnit GeographyUnit
        {
            get { return geographyUnit; }
            set { geographyUnit = value; }
        }

        public DistanceUnit LengthUnit
        {
            get { return lengthUnit; }
            set { lengthUnit = value; }
        }

        public Feature RoadFeature
        {
            get { return roadFeature; }
            set { roadFeature = value; }
        }

        public string RoadNameColumnName
        {
            get { return roadNameColumnName; }
            set { roadNameColumnName = value; }
        }

        public RouteSegment RoadSegment
        {
            get { return roadSegment; }
            set { roadSegment = value; }
        }

        public Feature RoadSegmentFeature
        {
            get { return roadSegmentFeature; }
            set { roadSegmentFeature = value; }
        }

        public string Length
        {
            get
            {
                double value = Math.Round(((LineBaseShape)RoadFeature.GetShape()).GetLength(GeographyUnit, lengthUnit), 2);
                return string.Format(CultureInfo.InvariantCulture, "{0} {1}", value, lengthUnit == DistanceUnit.Mile ? "mi" : "km");
            }
        }

        public string RoadName
        {
            get
            {
                string roadName = string.Empty;
                if (!string.IsNullOrEmpty(RoadNameColumnName))
                {
                    roadName = roadFeature.ColumnValues[RoadNameColumnName];
                }

                return roadName;
            }
        }

        public Uri DirectionImageUri
        {
            get
            {
                switch (Direction)
                {
                    case DrivingDirection.Back:
                        directionImageUriString = "back.png";
                        break;

                    case DrivingDirection.Forward:
                        directionImageUriString = "Forward.png";
                        break;

                    case DrivingDirection.Invalid:
                        directionImageUriString = "Invalid.png";
                        break;

                    case DrivingDirection.LeftForward:
                        directionImageUriString = "LeftForward.png";
                        break;

                    case DrivingDirection.Right:
                        directionImageUriString = "Right.png";
                        break;

                    case DrivingDirection.RightBack:
                        directionImageUriString = "RightBack.png";
                        break;

                    case DrivingDirection.RightForward:
                        directionImageUriString = "RightForward.png";
                        break;

                    case DrivingDirection.LeftBack:
                        directionImageUriString = "LeftBack.png";
                        break;

                    case DrivingDirection.Left:
                        directionImageUriString = "Left.png";
                        break;
                }

                return new Uri(string.Format(CultureInfo.InvariantCulture, directionImageUriTemplate, directionImageUriString), UriKind.RelativeOrAbsolute);
            }
        }
    }
}